@@ -11,7 +11,7 @@ use File::Spec;
 use Log::Any 0.14;
 use Log::Any::Adapter;
 
-our $VERSION = 0.43; # xVERSION
+our $VERSION = 0.44; # xVERSION
 
 our $ALT = "FWR";
 
@@ -36,12 +36,15 @@ my $is_daemon;
 
 # poor man's version of 5.10's //
 sub _ifdef {
-    my ($a, $b) = @_;
-    defined($a) ? $a : $b;
+    my $def = pop @_;
+    for (@_) {
+        return $_ if defined($_);
+    }
+    $def;
 }
 
-# m=multi, j=as json (except the last default)
-sub _ifdefmj {
+# j=as json (except the last default)
+sub _ifdefj {
     require JSON;
 
     my $def = pop @_;
@@ -84,7 +87,7 @@ sub _gen_appender_config {
     } elsif ($name =~ /^file/i) {
         $class = "Log::Dispatch::FileWriteRotate";
         my ($dir, $prefix) = $ospec->{path} =~ m!(.+)/(.+)!;
-        $dir //= "."; $prefix //= $ospec->{path};
+        $dir ||= "."; $prefix ||= $ospec->{path};
         $params->{dir}         = $dir;
         $params->{prefix}      = $prefix;
         $params->{suffix}      = $ospec->{suffix};
@@ -117,22 +120,75 @@ sub _gen_appender_config {
     );
 }
 
+sub _lit {
+    require Data::Dump;
+    Data::Dump::dump(shift);
+}
+
 sub _gen_l4p_config {
     my ($spec) = @_;
 
+    my @otypes = qw(file dir screen syslog);
+
+    # we use a custom perl code to implement filter_* specs.
+    my @fccode;
+    push @fccode, 'my %p = @_';
+    push @fccode, 'my $str';
+    for my $ospec (map { @{ $spec->{$_} } } @otypes) {
+        if (defined $ospec->{filter_text}) {
+            push @fccode, '$str = '._lit($ospec->{filter_text});
+            push @fccode, 'return 0 if $p{name} eq '._lit($ospec->{name}).
+                ' && index($_, $str) == -1';
+        }
+        if (defined $ospec->{filter_no_text}) {
+            push @fccode, '$str = '._lit($ospec->{filter_no_text});
+            push @fccode, 'return 0 if $p{name} eq '._lit($ospec->{name}).
+                ' && index($_, $str) > -1';
+        }
+        if (defined $ospec->{filter_citext}) {
+            push @fccode, '$str = '._lit($ospec->{filter_citext});
+            push @fccode, 'return 0 if $p{name} eq '._lit($ospec->{name}).
+                ' && !/\Q$str/io';
+        }
+        if (defined $ospec->{filter_no_citext}) {
+            push @fccode, '$str = '._lit($ospec->{filter_no_citext});
+            push @fccode, 'return 0 if $p{name} eq '._lit($ospec->{name}).
+                ' && /\Q$str/io';
+        }
+        if (defined $ospec->{filter_re}) {
+            push @fccode, '$str = '._lit($ospec->{filter_re});
+            push @fccode, 'return 0 if $p{name} eq '._lit($ospec->{name}).
+                ' && $_ !~ ' . (ref($ospec->{filter_re}) eq 'Regexp' ? '$str' : 'qr/$str/o');
+        }
+        if (defined $ospec->{filter_no_re}) {
+            push @fccode, '$str = '._lit($ospec->{filter_no_re});
+            push @fccode, 'return 0 if $p{name} eq '._lit($ospec->{name}).
+                ' && $_ =~ ' . (ref($ospec->{filter_re}) eq 'Regexp' ? '$str' : 'qr/$str/o');
+        }
+    }
+    push @fccode, "1";
+    my $fccode = join "; ", @fccode;
+
     my $filters_str = join(
         "",
-        "log4perl.filter.FilterOFF = Log::Log4perl::Filter::LevelRange\n",
-        "log4perl.filter.FilterOFF.LevelMin = TRACE\n",
-        "log4perl.filter.FilterOFF.LevelMax = FATAL\n",
-        "log4perl.filter.FilterOFF.AcceptOnMatch = false\n",
+        "log4perl.filter.FilterCustom = sub { $fccode }\n",
         "\n",
+        "log4perl.filter.FilterOFF0 = Log::Log4perl::Filter::LevelRange\n",
+        "log4perl.filter.FilterOFF0.LevelMin = TRACE\n",
+        "log4perl.filter.FilterOFF0.LevelMax = FATAL\n",
+        "log4perl.filter.FilterOFF0.AcceptOnMatch = false\n",
+        "\n",
+        "log4perl.filter.FilterOFF = Log::Log4perl::Filter::Boolean\n",
+        "log4perl.filter.FilterOFF.logic = FilterOFF0 && FilterCustom\n",
         map {join(
             "",
-            "log4perl.filter.Filter$_ = Log::Log4perl::Filter::LevelRange\n",
-            "log4perl.filter.Filter$_.LevelMin = $_\n",
-            "log4perl.filter.Filter$_.LevelMax = FATAL\n",
-            "log4perl.filter.Filter$_.AcceptOnMatch = true\n",
+            "log4perl.filter.Filter${_}0 = Log::Log4perl::Filter::LevelRange\n",
+            "log4perl.filter.Filter${_}0.LevelMin = $_\n",
+            "log4perl.filter.Filter${_}0.LevelMax = FATAL\n",
+            "log4perl.filter.Filter${_}0.AcceptOnMatch = true\n",
+            "\n",
+            "log4perl.filter.Filter$_ = Log::Log4perl::Filter::Boolean\n",
+            "log4perl.filter.Filter$_.logic = Filter${_}0 && FilterCustom\n",
             "\n",
         )} qw(FATAL ERROR WARN INFO DEBUG), # TRACE
     );
@@ -142,10 +198,7 @@ sub _gen_l4p_config {
     my %ospecs; # key = oname; this is just a shortcut to get ospec
 
     # 1. list all levels for each category and output
-    for my $ospec (@{ $spec->{dir} },
-                   @{ $spec->{file} },
-                   @{ $spec->{screen} },
-                   @{ $spec->{syslog} }) {
+    for my $ospec (map { @{ $spec->{$_} } } @otypes) {
         my $oname = $ospec->{name};
         $ospecs{$oname} = $ospec;
         $levels{$oname} = {};
@@ -234,6 +287,7 @@ sub _gen_l4p_config {
                 $filter = "Filter".uc($olevel);
             } else {
                 $apd_name = $oname;
+                $filter = "FilterCustom";
             }
             unless ($generated_appenders{$apd_name}++) {
                 $apd_str .= _gen_appender_config($ospec, $apd_name, $filter).
@@ -278,9 +332,9 @@ sub _init_log4perl {
 
     my $config_str = _gen_l4p_config($spec);
     if ($spec->{dump}) {
-        require Data::Dumper;
+        require Data::Dump;
         print "Log::Any::App configuration:\n",
-            Data::Dumper->new([$spec])->Terse(1)->Dump;
+            Data::Dump::dump($spec);
         print "Log4perl configuration: <<EOC\n", $config_str, "EOC\n";
     }
 
@@ -342,7 +396,7 @@ sub _parse_opts {
 
     my $spec = {};
 
-    $spec->{log} = $ENV{LOG} // 1;
+    $spec->{log} = _ifdef($ENV{LOG}, 1);
     if (defined $opts{log}) {
         $spec->{log} = $opts{log};
         delete $opts{log};
@@ -372,7 +426,7 @@ sub _parse_opts {
     }
     delete $opts{level};
 
-    $spec->{category_alias} = _ifdefmj($ENV{LOG_CATEGORY_ALIAS}, {});
+    $spec->{category_alias} = _ifdefj($ENV{LOG_CATEGORY_ALIAS}, {});
     if (defined $opts{category_alias}) {
         die "category_alias must be a hashref"
             unless ref($opts{category_alias}) eq 'HASH';
@@ -410,6 +464,37 @@ sub _parse_opts {
         delete $opts{dump};
     }
 
+    $spec->{filter_text} = $ENV{LOG_FILTER_TEXT};
+    if (defined $opts{filter_text}) {
+        $spec->{filter_text} = $opts{filter_text};
+        delete $opts{filter_text};
+    }
+    $spec->{filter_no_text} = $ENV{LOG_FILTER_NO_TEXT};
+    if (defined $opts{filter_no_text}) {
+        $spec->{filter_no_text} = $opts{filter_no_text};
+        delete $opts{filter_no_text};
+    }
+    $spec->{filter_citext} = $ENV{LOG_FILTER_CITEXT};
+    if (defined $opts{filter_citext}) {
+        $spec->{filter_citext} = $opts{filter_citext};
+        delete $opts{filter_citext};
+    }
+    $spec->{filter_no_citext} = $ENV{LOG_FILTER_NO_CITEXT};
+    if (defined $opts{filter_no_citext}) {
+        $spec->{filter_no_citext} = $opts{filter_no_citext};
+        delete $opts{filter_no_citext};
+    }
+    $spec->{filter_re} = $ENV{LOG_FILTER_RE};
+    if (defined $opts{filter_re}) {
+        $spec->{filter_re} = $opts{filter_re};
+        delete $opts{filter_re};
+    }
+    $spec->{filter_no_re} = $ENV{LOG_FILTER_NO_RE};
+    if (defined $opts{filter_no_re}) {
+        $spec->{filter_no_re} = $opts{filter_no_re};
+        delete $opts{filter_no_re};
+    }
+
     $spec->{file} = [];
     _parse_opt_file($spec, _ifdef($opts{file}, ($0 ne '-e' ? 1:0)));
     delete $opts{file};
@@ -429,11 +514,11 @@ sub _parse_opts {
     if (keys %opts) {
         die "Unknown option(s) ".join(", ", keys %opts)." Known opts are: ".
             "log, name, level, category_level, category_alias, dump, init, ".
-                "file, dir, screen, syslog";
+                "filter_{,no_}{text,citext,re}, file, dir, screen, syslog";
     }
 
   END_PARSE_OPTS:
-    #use Data::Dumper; print Dumper $spec;
+    #use Data::Dump; dd $spec;
     $spec;
 }
 
@@ -522,9 +607,9 @@ sub _default_file {
     }
     return {
         level => $level,
-        category_level => _ifdefmj($ENV{FILE_LOG_CATEGORY_LEVEL},
-                                   $ENV{LOG_CATEGORY_LEVEL},
-                                   $spec->{category_level}),
+        category_level => _ifdefj($ENV{FILE_LOG_CATEGORY_LEVEL},
+                                  $ENV{LOG_CATEGORY_LEVEL},
+                                  $spec->{category_level}),
         path => $> ? File::Spec->catfile(File::HomeDir->my_home, "$spec->{name}.log") :
             "/var/log/$spec->{name}.log", # XXX and on Windows?
         max_size => undef,
@@ -534,6 +619,13 @@ sub _default_file {
         category => '',
         pattern_style => _set_pattern_style('daemon'),
         pattern => undef,
+
+        filter_text      => _ifdef($ENV{FILE_LOG_FILTER_TEXT}, $spec->{filter_text}),
+        filter_no_text   => _ifdef($ENV{FILE_LOG_FILTER_NO_TEXT}, $spec->{filter_no_text}),
+        filter_citext    => _ifdef($ENV{FILE_LOG_FILTER_CITEXT}, $spec->{filter_citext}),
+        filter_no_citext => _ifdef($ENV{FILE_LOG_FILTER_NO_CITEXT}, $spec->{filter_no_citext}),
+        filter_re        => _ifdef($ENV{FILE_LOG_FILTER_RE}, $spec->{filter_re}),
+        filter_no_re     => _ifdef($ENV{FILE_LOG_FILTER_NO_RE}, $spec->{filter_no_re}),
     };
 }
 
@@ -573,7 +665,7 @@ sub _default_dir {
     }
     return {
         level => $level,
-        category_level => _ifdefmj($ENV{DIR_LOG_CATEGORY_LEVEL},
+        category_level => _ifdefj($ENV{DIR_LOG_CATEGORY_LEVEL},
                                    $ENV{LOG_CATEGORY_LEVEL},
                                    $spec->{category_level}),
         path => $> ? File::Spec->catfile(File::HomeDir->my_home, "log", $spec->{name}) :
@@ -585,6 +677,13 @@ sub _default_dir {
         pattern_style => _set_pattern_style('plain'),
         pattern => undef,
         filename_pattern => undef,
+
+        filter_text      => _ifdef($ENV{DIR_LOG_FILTER_TEXT}, $spec->{filter_text}),
+        filter_no_text   => _ifdef($ENV{DIR_LOG_FILTER_NO_TEXT}, $spec->{filter_no_text}),
+        filter_citext    => _ifdef($ENV{DIR_LOG_FILTER_CITEXT}, $spec->{filter_citext}),
+        filter_no_citext => _ifdef($ENV{DIR_LOG_FILTER_NO_CITEXT}, $spec->{filter_no_citext}),
+        filter_re        => _ifdef($ENV{DIR_LOG_FILTER_RE}, $spec->{filter_re}),
+        filter_no_re     => _ifdef($ENV{DIR_LOG_FILTER_NO_RE}, $spec->{filter_no_re}),
     };
 }
 
@@ -612,13 +711,20 @@ sub _default_screen {
         color => _ifdef($ENV{COLOR}, (-t STDOUT)),
         stderr => 1,
         level => $level,
-        category_level => _ifdefmj($ENV{SCREEN_LOG_CATEGORY_LEVEL},
+        category_level => _ifdefj($ENV{SCREEN_LOG_CATEGORY_LEVEL},
                                    $ENV{LOG_CATEGORY_LEVEL},
                                    $spec->{category_level}),
         category => '',
         pattern_style => _set_pattern_style(
             $ENV{LOG_ELAPSED_TIME_IN_SCREEN} ? 'script_short' : 'plain_nl'),
         pattern => undef,
+
+        filter_text      => _ifdef($ENV{SCREEN_LOG_FILTER_TEXT}, $spec->{filter_text}),
+        filter_no_text   => _ifdef($ENV{SCREEN_FILTER_NO_TEXT}, $spec->{filter_no_text}),
+        filter_citext    => _ifdef($ENV{SCREEN_FILTER_CITEXT}, $spec->{filter_citext}),
+        filter_no_citext => _ifdef($ENV{SCREEN_FILTER_NO_CITEXT}, $spec->{filter_no_citext}),
+        filter_re        => _ifdef($ENV{SCREEN_FILTER_RE}, $spec->{filter_re}),
+        filter_no_re     => _ifdef($ENV{SCREEN_FILTER_NO_RE}, $spec->{filter_no_re}),
     };
 }
 
@@ -639,7 +745,7 @@ sub _default_syslog {
     }
     return {
         level => $level,
-        category_level => _ifdefmj($ENV{SYSLOG_LOG_CATEGORY_LEVEL},
+        category_level => _ifdefj($ENV{SYSLOG_LOG_CATEGORY_LEVEL},
                                    $ENV{LOG_CATEGORY_LEVEL},
                                    $spec->{category_level}),
         ident => $spec->{name},
@@ -647,6 +753,13 @@ sub _default_syslog {
         pattern_style => _set_pattern_style('syslog'),
         pattern => undef,
         category => '',
+
+        filter_text      => _ifdef($ENV{SYSLOG_LOG_FILTER_TEXT}, $spec->{filter_text}),
+        filter_no_text   => _ifdef($ENV{SYSLOG_FILTER_NO_TEXT}, $spec->{filter_no_text}),
+        filter_citext    => _ifdef($ENV{SYSLOG_FILTER_CITEXT}, $spec->{filter_citext}),
+        filter_no_citext => _ifdef($ENV{SYSLOG_FILTER_NO_CITEXT}, $spec->{filter_no_citext}),
+        filter_re        => _ifdef($ENV{SYSLOG_FILTER_RE}, $spec->{filter_re}),
+        filter_no_re     => _ifdef($ENV{SYSLOG_FILTER_NO_RE}, $spec->{filter_no_re}),
     };
 }
 
@@ -712,7 +825,7 @@ sub _check_level {
 
 sub _set_level {
     my ($prefix, $which, $spec) = @_;
-    #use Data::Dumper; print Dumper $spec;
+    #use Data::Dump; dd $spec;
     my $p_ = $prefix ? "${prefix}_" : "";
     my $P_ = $prefix ? uc("${prefix}_") : "";
     my $F_ = $prefix ? ucfirst("${prefix}_") : "";
@@ -900,13 +1013,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Log::Any::App - An easy way to use Log::Any in applications
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -1034,7 +1149,7 @@ To change level from inside the script:
 
  use Log::Any::App '$log', -level => 'debug';
 
-This is useful if you want a fixed level that cannot be overriden by other
+This is useful if you want a fixed level that cannot be overridden by other
 mechanisms (since setting level using import argument has the highest priority).
 But oftentimes what you want is changing level without modifying the script
 itself. Thereby, just write:
@@ -1106,7 +1221,7 @@ basis:
      # and so on
  }
 
-If a per-output level is not specifed, it will default to the general log level.
+If a per-output level is not specified, it will default to the general log level.
 
 =head2 Enabling/disabling output
 
@@ -1118,7 +1233,7 @@ or:
 
  use Log::Any::App '$log', -screen => {level=>'off'};
 
-and this won't allow the output to be reenabled from outside the script. However
+and this won't allow the output to be re-enabled from outside the script. However
 if you do this:
 
  use Log::Any::App;
@@ -1415,6 +1530,35 @@ C<$DEBUG> or C<$debug>.
 
 If everything fails, it defaults to 'warn'.
 
+=item -filter_text => STR
+
+Only show log lines matching STR. Default from C<LOG_FILTER_TEXT> environment.
+
+=item -filter_no_text => STR
+
+Only show log lines not matching STR. Default from C<LOG_FILTER_NO_TEXT>
+environment.
+
+=item -filter_citext => STR
+
+Only show log lines matching STR (case insensitive). Default from
+C<LOG_FILTER_CITEXT> environment.
+
+=item -filter_no_citext => STR
+
+Only show log lines not matching STR (case insensitive). Default from
+C<LOG_FILTER_NO_CITEXT> environment.
+
+=item -filter_re => RE
+
+Only show log lines matching regex pattern RE. Default from C<LOG_FILTER_RE>
+environment.
+
+=item -filter_no_re => RE
+
+Only show log lines not matching regex pattern RE. Default from
+C<LOG_FILTER_NO_RE> environment.
+
 =item -file => 0 | 1|yes|true | PATH | {opts} | [{opts}, ...]
 
 Specify output to one or more files, using L<Log::Dispatch::FileWriteRotate>.
@@ -1430,7 +1574,9 @@ Log::Dispatch::FileWriteRotate's constructor), C<period> (will be passed to
 Log::Dispatch::FileWriteRotate's constructor), C<buffer_size> (will be passed to
 Log::Dispatch::FileWriteRotate's constructor), C<category> (a string of ref to
 array of strings), C<category_level> (a hashref, similar to -category_level),
-C<pattern_style> (see L<"PATTERN STYLES">), C<pattern> (Log4perl pattern).
+C<pattern_style> (see L<"PATTERN STYLES">), C<pattern> (Log4perl pattern),
+C<filter_text>, C<filter_no_text>, C<filter_citext>, C<filter_no_citext>,
+C<filter_re>, C<filter_no_re>.
 
 If the argument is an arrayref, it is assumed to be specifying multiple files,
 with each element of the array as a hashref.
@@ -1472,7 +1618,8 @@ C<max_age> (maximum age of files to keep, in seconds, undef means unlimited).
 C<histories> (number of old files to keep, excluding the current file),
 C<category>, C<category_level> (a hashref, similar to -category_level),
 C<pattern_style> (see L<"PATTERN STYLES">), C<pattern> (Log4perl pattern),
-C<filename_pattern> (pattern of file name).
+C<filename_pattern> (pattern of file name), C<filter_text>, C<filter_no_text>,
+C<filter_citext>, C<filter_no_citext>, C<filter_re>, C<filter_no_re>.
 
 If the argument is an arrayref, it is assumed to be specifying multiple
 directories, with each element of the array as a hashref.
@@ -1506,7 +1653,8 @@ the hashref must be one of: C<color> (default is true, set to 0 to turn off
 color), C<stderr> (default is true, set to 0 to log to stdout instead),
 C<level>, C<category>, C<category_level> (a hashref, similar to
 -category_level), C<pattern_style> (see L<"PATTERN STYLE">), C<pattern>
-(Log4perl string pattern).
+(Log4perl string pattern), C<filter_text>, C<filter_no_text>, C<filter_citext>,
+C<filter_no_citext>, C<filter_re>, C<filter_no_re>.
 
 How Log::Any::App determines defaults for screen logging:
 
@@ -1532,7 +1680,9 @@ argument is a true value that matches /^(1|yes|true)$/i, syslog logging will be
 turned on with default level, ident, etc. If the argument is a hashref, then the
 keys of the hashref must be one of: C<level>, C<ident>, C<facility>,
 C<category>, C<category_level> (a hashref, similar to -category_level),
-C<pattern_style> (see L<"PATTERN STYLES">), C<pattern> (Log4perl pattern).
+C<pattern_style> (see L<"PATTERN STYLES">), C<pattern> (Log4perl pattern),
+C<filter_text>, C<filter_no_text>, C<filter_citext>, C<filter_no_citext>,
+C<filter_re>, C<filter_no_re>.
 
 How Log::Any::App determines defaults for syslog logging:
 
@@ -1673,6 +1823,20 @@ Below is summary of environment variables used.
 
  LOG_ELAPSED_TIME_IN_SCREEN (bool)
 
+=head2 Filtering
+
+ LOG_FILTER_TEXT (str)
+ LOG_FILTER_NO_TEXT (str)
+ LOG_FILTER_CITEXT (str)
+ LOG_FILTER_NO_CITEXT (str)
+ LOG_FILTER_RE (str)
+ LOG_FILTER_NO_RE (str)
+
+=head2 Per-output filtering
+
+ {FILE,DIR,SCREEN,SYSLOG}_LOG_FILTER_TEXT (str)
+ and so on
+
 =head2 Extra things to log
 
 =over
@@ -1768,7 +1932,7 @@ some incompatibilities, please read this section carefully.
 As I I<love> specifying log options from environment, I will make I<every>
 init() options configurable from outside the script
 (environment/command-line/control file). Of course, init() arguments still take
-precedence for authors that do not want some/all options to be overriden from
+precedence for authors that do not want some/all options to be overridden from
 outside.
 
 =item * Reorganization of command-line/environment names
@@ -1824,13 +1988,29 @@ Some alternative logging modules: L<Log::Dispatchouli> (based on
 L<Log::Dispatch>), L<Log::Fast>, L<Log::Log4perl::Tiny>. Really, there are 7,451
 of them (roughly one third of CPAN) at the time of this writing.
 
+=head1 HOMEPAGE
+
+Please visit the project's homepage at L<https://metacpan.org/release/Alt-Log-Any-App-FWR>.
+
+=head1 SOURCE
+
+Source repository is at L<https://github.com/sharyanto/perl-Alt-Log-Any-App-FWR>.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Alt-Log-Any-App-FWR>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Steven Haryanto.
+This software is copyright (c) 2014 by Steven Haryanto.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
